@@ -11,7 +11,7 @@ use MooX::Types::MooseLike::Base qw(:all);
 use PDF::Table;
 use PDF::TextBlock 0.13;
 
-our $VERSION = '0.10';
+our $VERSION = '0.20';
 
 =head1 NAME
 
@@ -94,6 +94,15 @@ module. Also, it is built on either PDF::Builder or PDF::API2, and provides
 access to that object, so content can also be added directly using that,
 thereby providing any powerful features required.
 
+B<Updates in v0.20> Note that version 0.20 contains a number breaking changes
+to improve the default layout and spacing of a page. This better ensures that
+content added to a page "just works" in terms of its layout, without needing
+tweaks to its spacing. For example, headers have better spacing above and below
+by default. This means that PDFs produced with this version will be laid out
+differently to those produced with earlier versions. In the main, old code
+should be able to be updated by simply removing any manual spacing fudges (e.g.
+manual spacing for headers).
+
 =head1 METHODS
 
 =cut
@@ -190,11 +199,6 @@ sub add_page
     $page->mediabox(0, 0, $self->width, $self->height);
     $self->_set_page($page);
     $self->_set__y($self->_y_start_default); # Reset y cursor
-    if ($self->logo && $self->pdf->pages > 1)
-    {
-        $self->_down($self->logo_height);
-        $self->_down($self->logo_padding);
-    }
     # Flag that we have just started a new page. Because text is positioned from
     # its bottom-left corner, we will need to move the cursor down further to
     # account for the font size of the text, but we don't know that yet.
@@ -310,6 +314,49 @@ has margin => (
     default => 40,
 );
 
+=head2 margin_top
+
+Sets or returns the top margin. Defaults to the margin + top_padding +
+room for the header (if defined) + room for the logo (if defined).
+
+=cut
+
+has margin_top => (
+    is      => 'lazy',
+    isa     => Int,
+);
+
+sub _build_margin_top
+{   my $self = shift;
+    my $size = $self->margin + $self->top_padding;
+    $size += 15 if $self->header; # Arbitrary number to allow 10px of header text
+    if ($self->logo)
+    {
+        $size += $self->logo_height;
+        $size += $self->logo_padding;
+    }
+    return $size;
+};
+
+=head2 margin_bottom
+
+Sets or returns the bottom margin. Defaults to the margin + room for the
+footer.
+
+=cut
+
+has margin_bottom => (
+    is      => 'lazy',
+    isa     => Int,
+);
+
+sub _build_margin_bottom
+{   my $self = shift;
+    my $size = $self->margin;
+    $size += 15; # Arbitrary number to allow 10px of footer text
+    return $size;
+};
+
 =head2 top_padding
 
 Sets or returns the top padding (additional to the margin). Default 0.
@@ -500,9 +547,7 @@ has _y => (
 
 sub _y_start_default
 {   my $self = shift;
-    my $start = $self->height - $self->margin - $self->top_padding;
-    $start -= 15 if $self->header; # Arbitrary number to allow 10px of header text
-    $start;
+    return $self->height - $self->margin_top;
 }
 
 =head2 heading($text, %options)
@@ -523,32 +568,71 @@ C<n> is the amount (in points) to indent the text, B<default 0>
 
 =item topmargin I<n>
 
-C<n> is the amount (in points) of vertical skip for the margin I<above> the heading, B<default 0>
+C<n> is the amount (in points) of vertical skip for the margin I<above> the
+heading, B<default:> calculated automatically based on font size
 
 =item bottommargin I<n>
 
-C<n> is the amount (in points) of vertical skip for the margin I<below> the heading, B<default 10>
+C<n> is the amount (in points) of vertical skip for the margin I<below> the
+heading, B<default:> calculated automatically based on the font size
 
 =back
 
 =cut
 
+# Return the line height based on a font size, with optional ratio
+sub _line_height {
+    my ($self, $size, $ratio) = @_;
+    $size * ($ratio || 1.5);
+}
+
+# Return the spacing above/below a line based on font size and line height
+sub _line_spacing {
+    my ($self, $size) = @_;
+    my $spacing = $self->_line_height($size);
+    ($spacing - $size) / 2;
+}
+
 sub heading
 {   my ($self, $string, %options) = @_;
-    $self->_down($options{topmargin}) if $options{topmargin};
+
+    $self->page; # Ensure that page is built and cursor adjusted for first use
+
     $self->add_page if $self->_y < 150; # Make sure there is room for following paragraph text
     my $size = $options{size} || 16;
-    if ($self->is_new_page)
+
+    if ($options{topmargin}) {
+        # Always let override take precedence
+        $self->_down($options{topmargin}) if $options{topmargin};
+    }
+    elsif ($self->is_new_page)
     {
+        # If a new page then move down just enough to fit in the font size
         $self->_down($size);
         $self->_set_is_new_page(0);
     }
+    else {
+        # Default to top margin based on font size, with slightly higher
+        # spacing ratio than normal text
+        $self->_down($self->_line_height($size, 1.8));
+    }
     my $tb  = PDF::TextBlock->new({
         pdf  => $self->pdf,
         page => $self->page,
         x    => $self->_x + ($options{indent} || 0),
         y    => $self->_y,
+        lead => $self->_line_height($size, 1.6),
         fonts => {
+            # Workaround a bug in PDF::TextBlock which defines word spacing
+            # based on the default font. This can lead to spacing that is too
+            # small if only using a bold font with a large font size. Define
+            # the default font to be the same as the bold font that we will
+            # use.
+            default => PDF::TextBlock::Font->new({
+                pdf  => $self->pdf,
+                font => $self->fontbold,
+                size => $size,
+            }),
             b => PDF::TextBlock::Font->new({
                 pdf  => $self->pdf,
                 font => $self->fontbold,
@@ -558,9 +642,13 @@ sub heading
     });
     $tb->text('<b>'.$string.'</b>');
     my ($endw, $ypos) = $tb->apply;
-    $self->_set__y($ypos);
-    my $bottommargin = defined $options{bottommargin} ? $options{bottommargin} : 10; # Allow zero
-    $self->_down($bottommargin) if $bottommargin;
+    $self->_set__y($ypos + $self->_line_height($size)); # Move cursor back to end of last line printed
+
+    # Unless otherwise defined, add a bottom margin relative to the font size,
+    # but smaller than the top margin
+    my $bottommargin = defined $options{bottommargin} ? $options{bottommargin} : $self->_line_height($size, 0.4);;
+
+    $self->_down($bottommargin);
 }
 
 =head2 text($text, %options)
@@ -591,11 +679,19 @@ sub text
     my $size = $options{size} || 10;
     my $color = $options{color} || 'black';
 
+    $self->page; # Ensure that page is built and cursor adjusted for first use
+
     if ($self->is_new_page)
     {
-        $self->_down($size);  # TBD using font size as line height?
         $self->_set_is_new_page(0);
     }
+    else {
+        # Only create spacing if below other content
+        $self->_down($self->_line_spacing($size));
+    }
+
+    # Line spacing already accounted for above, now allow enough room for actual font size
+    $self->_down($size);
 
     my $tb  = PDF::TextBlock->new({
         pdf   => $self->pdf,
@@ -603,7 +699,8 @@ sub text
         x     => $self->_x + ($options{indent} || 0),
         y     => $self->_y,
         w     => $self->_width_print,
-        h     => $self->_y - $self->margin - 30,
+        h     => $self->_y - $self->margin_bottom,
+        lead  => $self->_line_height($size),
         align => 'left',
         fonts => {
             default => PDF::TextBlock::Font->new({
@@ -622,40 +719,55 @@ sub text
     });
     while (1)
     {
-        # For reasons I do not understand, $string manages to gain newlines
-        # between the end and beginning of this loop. Chop them off, and end if
-        # there's nothing left
-        $string =~ s/\s+$//;
-        !$string and last;
-        $tb->text($string);
-        my $endw; my $ypos;
-        my $string_before = $string;
-        ($endw, $ypos, $string) = $tb->apply;
-        # Check whether no text has been added to the page. This happens if the
-        # word is too long. If so, warn, chop-off and retry, otherwise an
-        # infinite loop occurs. Ideally the word would be broken - issue will
-        # be raised in PDF::TextBlock to see if this is possible.
-        if ($string_before eq $string)
+        # First check whether there is any room on the page for the text. If
+        # not, start a new page. This code is copied directly from the same
+        # check in PDF::TextBlock, with 15 being the default lead. We can no
+        # longer rely on PDF::TextBlock returning the same $string to know to
+        # insert a new page, as we now use that to check for words that are too
+        # long
+        if ($tb->y >= $tb->y - $tb->h + 15) # Same condition as PDF::TextBlock
         {
-            carp "Unable to fit text onto line: $string";
-            # If no more breaks then skip
-            last if $string !~ /\s/;
-            # Otherwise start from after next break
-            $string =~ s/\S+\s//;
+            # For reasons I do not understand, $string manages to gain newlines
+            # between the end and beginning of this loop. Chop them off, and end if
+            # there's nothing left
+            $string =~ s/\s+$//;
+            !$string and last;
             $tb->text($string);
+            my $endw; my $ypos;
+            my $string_before = $string;
             ($endw, $ypos, $string) = $tb->apply;
+            # Check whether no text has been added to the page. This happens if the
+            # word is too long. If so, warn, chop-off and retry, otherwise an
+            # infinite loop occurs. Ideally the word would be broken - issue will
+            # be raised in PDF::TextBlock to see if this is possible.
+            if ($string_before eq $string)
+            {
+                carp "Unable to fit text onto line: $string";
+                # If no more breaks then skip
+                last if $string !~ /\s/;
+                # Otherwise start from after next break
+                $string =~ s/\S+\s//;
+                $tb->text($string);
+                ($endw, $ypos, $string) = $tb->apply;
+            }
+
+            # Set y cursor to be where the textblock finished, but move cursor
+            # back up to remove new line
+            $self->_set__y($ypos + $tb->lead);
+            # Now shift down the actual line spacing distance
+            $self->_down($self->_line_spacing($size));
+            last unless $string; # while loop does not work with $string
         }
-        $self->_set__y($ypos);
-        last unless $string; # while loop does not work with $string
         $self->add_page;
-        $self->_down($size); # TBD again, font size as line height?
+        $self->_down($size);
         $tb  = PDF::TextBlock->new({
             pdf   => $self->pdf,
             page  => $self->page,
             x     => $self->_x,
             y     => $self->_y,
             w     => $self->_width_print,
-            h     => $self->_y - $self->margin - 30,
+            h     => $self->_y - $self->margin_bottom,
+            lead  => $self->_line_height($size),
             align => 'left',
             fonts => {
                 default => PDF::TextBlock::Font->new({
@@ -673,8 +785,6 @@ sub text
             },
         });
     }
-
-    $self->_down(5);
 }
 
 =head2 table(%options)
@@ -688,27 +798,31 @@ PDF::Table.
 sub table
 {   my ($self, %options) = @_;
 
-    # Move onto new page if little space left on this one
-    $self->add_page if $self->_y < 100;
+    $self->page; # Ensure that page is built and cursor adjusted for first use
+
+    # Move onto new page if little space left on this one.
+    # TODO Change arbitary "60" to something calculated? Needs to be able to
+    # fit header and one row as a minimum.
+    $self->add_page if $self->_y < 60 + $self->margin_bottom;
 
     my $table = PDF::Table->new;
 
     my $data = delete $options{data};
 
+    # Create spacing above and below table based on the line spacing for text
+    # of 10 points
+    $self->_down($self->_line_height(10));
+
     # Keep separate so easy to dump for debug
-    my $hf_space = 0;
-    $hf_space += 40 if $self->footer;
-    $hf_space += 40 if $self->header;
     my %dimensions = (
-        next_h    => $self->height - $self->margin - ($self->height - $self->_y_start_default) - $self->margin,
+        next_h    => $self->_y_start_default - $self->margin_bottom,
         x         => $self->_x,
         w         => $self->_width_print,
         font_size => 10,
         padding   => 5,
-        # start_y deprecated, change soon to y. start_h deprecated, change to h
         y         => $self->_y,
-        h         => $self->height - ($self->height - $self->_y) - $self->margin - $hf_space,
-        next_y    => $self->height - $self->margin - ($self->height - $self->_y_start_default),
+        h         => $self->_y - $self->margin_bottom,
+        next_y    => $self->_y_start_default,
     );
     my ($final_page, $number_of_pages, $final_y) = $table->table(
         $self->pdf,
@@ -731,8 +845,10 @@ sub table
         },
         %options,
     );
+    $self->clear_new_page;
     $self->_set__y($final_y);
-    $self->_down(20);
+    # As above, padding below table
+    $self->_down($self->_line_height(10));
 }
 
 sub _image_type
@@ -751,7 +867,11 @@ Add an image. Options available are:
 
 =item scaling I<n>
 
-    C<n> is the scaling factor for the image, B<default 0.5> (50%)
+C<n> is the scaling factor for the image, B<default 0.5> (50%)
+
+=item alignment I<name>
+
+C<name> is the horizontal alignment, B<default center>
 
 =back
 
@@ -763,15 +883,21 @@ sub image
     my $info = image_info($file);
     my $width = $info->{width};
     my $height = $info->{height};
+    my $alignment = $options{alignment} || 'center';
     $height = $height * $scaling;
     $width  = $width * $scaling;
     $self->add_page if $height > $self->_y;
     $self->_down($height);
-    my $x = ($self->width / 2) - ($width / 2);
+    my $x = $alignment eq 'left'
+        ? $self->margin
+        : $alignment eq 'right'
+        ? $self->width - $self->margin - $width
+        : ($self->width / 2) - ($width / 2);
     my $type = lc 'image_'._image_type($file);
     my $image = $self->pdf->$type($file);
     my $gfx = $self->page->gfx;
     $gfx->image($image, $x, $self->_y, $scaling);
+    $self->clear_new_page;
 }
 
 =head2 content
